use std::process::Command;
use clap::{Parser, Subcommand};
use anyhow::Result;

#[derive(Parser)]
#[command(name = "thinkfanctl")]
#[command(about = "ThinkPad fan control utility")]
struct Args {
    #[command(subcommand)]
    command: Commands,
}

#[derive(Subcommand)]
enum Commands {
    Start,
    Stop,
    Restart,
    Status,
    Level { level: u8 },
    Auto,
    Config { path: String },
}

fn main() -> Result<()> {
    let args = Args::parse();
    
    match args.command {
        Commands::Start => start_daemon(),
        Commands::Stop => stop_daemon(),
        Commands::Restart => restart_daemon(),
        Commands::Status => show_status(),
        Commands::Level { level } => set_level(level),
        Commands::Auto => set_auto(),
        Commands::Config { path } => set_config(path),
    }
}

fn start_daemon() -> Result<()> {
    let status = Command::new("systemctl")
        .args(["start", "thinkfand"])
        .status()?;
    
    if status.success() {
        println!("Daemon started");
    } else {
        println!("Failed to start daemon");
    }
    Ok(())
}

fn stop_daemon() -> Result<()> {
    let status = Command::new("systemctl")
        .args(["stop", "thinkfand"])
        .status()?;
    
    if status.success() {
        println!("Daemon stopped");
    } else {
        println!("Failed to stop daemon");
    }
    Ok(())
}

fn restart_daemon() -> Result<()> {
    let status = Command::new("systemctl")
        .args(["restart", "thinkfand"])
        .status()?;
    
    if status.success() {
        println!("Daemon restarted");
    } else {
        println!("Failed to restart daemon");
    }
    Ok(())
}

fn show_status() -> Result<()> {
    let output = Command::new("systemctl")
        .args(["status", "thinkfand"])
        .output()?;
    
    println!("{}", String::from_utf8_lossy(&output.stdout));
    Ok(())
}

fn set_level(level: u8) -> Result<()> {
    if level > 7 {
        return Err(anyhow::anyhow!("Level must be 0-7"));
    }
    
    let status = Command::new("thinkfand")
        .args(["-l", &level.to_string()])
        .status()?;
    
    if status.success() {
        println!("Fan level set to {}", level);
    } else {
        println!("Failed to set fan level");
    }
    Ok(())
}

fn set_auto() -> Result<()> {
    let status = Command::new("thinkfand")
        .arg("-a")
        .status()?;
    
    if status.success() {
        println!("Fan set to auto mode");
    } else {
        println!("Failed to set auto mode");
    }
    Ok(())
}

fn set_config(path: String) -> Result<()> {
    let status = Command::new("cp")
        .args([&path, "/etc/thinkfan.conf"])
        .status()?;
    
    if status.success() {
        println!("Config updated");
        restart_daemon()?;
    } else {
        println!("Failed to update config");
    }
    Ok(())
} 